#!perl 

use Win32::AdminMisc;
use Win32::NetAdmin;
use Getopt::Long;

my ($machine,$domain,@users,%attribs);

GetOptions("d=s" => \$domain);

$machine = Win32::NodeName();

$domain =Win32::DomainName()
    unless (defined($domain));

if (@ARGV)
{
    @users = @ARGV;
}
else
{
  Win32::NetAdmin::GetUsers(Win32::AdminMisc::GetDC($domain),
                            '',\@users);
}

if (@users)
{
    print "Domain: $domain\n";
}

for $user (sort @users)
{
    unless(Win32::AdminMisc::UserGetMiscAttributes($domain,
                                                   $user,
                                                   \%attribs))
    {
        opstatus();
    }
    else
    {
        if ($attribs{"USER_PASSWORD_EXPIRED"} == 1)
        {
            print "Error: $user : Password has expired\n";
        }
        if ($machine eq Win32::AdminMisc::GetDC($domain))
        {
            unless(-d $attribs{"USER_HOME_DIR"})
            {
                print("Error: $user : Home directory, ", 
                      $attribs{"USER_HOME_DIR"},
                      " does not exist\n");
            }
        }
        if ($user !~ /Administrator/)
        {
            if ($attribs{"USER_PRIV"} == 2)
            {
                print("Warning: $user : User has ",
                      "administrator privileges\n");
            }
            else
            {
                if ($attribs{"USER_AUTH_FLAGS"} & 1)
                {
                    print("Warning: $user : User does not have ",
                          "administrator privileges but does ",
                          "have Print Admin privileges\n");
                }
                if ($attribs{"USER_AUTH_FLAGS"} & 4)
                {
                    print("Warning: $user : User does not have ",
                          "administrator privileges but does ",
                          "have Server Admin privileges\n");
                }
                if ($attribs{"USER_AUTH_FLAGS"} & 8)
                {
                    print("Warning: $user : User does not have ",
                          "administrator privileges but does ",
                          "have Accounts Admin privileges\n");
                }
            }
        }
        if ($attribs{"USER_PASSWORD_AGE"} > (31*24*60*60))
        {
            print("Error: $user : Password is more than ",
                  "one month (31 days) old\n");
        }
        if ($attribs{"USER_BAD_PW_COUNT"} > 0)
        {
            print("Warning: $user : User has had ", 
                  $attribs{"USER_BAD_PW_COUNT"}, 
                  " bad password attempts\n");
        }
    }
}

sub opstatus
{
    my $error=Win32::GetLastError();
    if ($error ne 0) 
    {
        print("\nERROR: ", Win32::FormatMessage($error),"\n");
    }
}
