#!perl -w
use Win32::Service;
use Getopt::Long;

my ($upserv,$downserv,$server,%services);

GetOptions("u=s" => \$upserv,
           "d=s" => \$downserv
           );
if (@ARGV) 
{
    $server = $ARGV[0];
}
else
{
    $server = Win32::NodeName();
}

Win32::Service::GetServices($server,\%services);

print "Server $server\n";

if ($upserv)
{
    startservices(split(/,/,$upserv));
}
elsif ($downserv)
{
    stopservices(split(/,/,$downserv));
}
else
{
    listservices();
}

sub startservices
{
    my (@servicelist) = @_;
    for my $service (@servicelist)
    {
        if (defined($services{$service}))
        {
            print "Starting the $service service...";
            Win32::Service::StartService($server,$service);
            opstatus("start $service");
            sleep 5;
            print "$service: ",chkstatus($service);
        }
        else
        {
            print "ERROR: Service $service does not exist\n";
        }
    }
}

sub stopservices
{
    my (@servicelist) = @_;
    for my $service (@servicelist)
    {
        if (defined($services{$service}))
        {
            print "Stopping the $service service...";
            Win32::Service::StopService($server,$service);
            opstatus("stop $service");
            sleep 5;
            print "$service: ",chkstatus($service);
        }
        else
        {
            print "ERROR: Service $service does not exist\n";
        }
    }
}

sub opstatus
{
    my ($opname) = @_;
    my $error=Win32::GetLastError();
    if ($error ne 0) 
    {
        print("\n$opname: ", Win32::FormatMessage($error),"\n");
    }
}

sub chkstatus
{
    my ($service) = @_;
    my %status;
    Win32::Service::GetStatus($server,$service,\%status);   
    if (defined($status{CurrentState}) 
        and $status{CurrentState} eq 4)
    {
        return "Started";
    }
    else
    {
        return "Stopped";
    }
}

sub listservices
{
    for my $service (sort keys %services)
    {
        print "$service: ",chkstatus($service),"\n";
    }
}
